/*
 * Copyright 2011-13 Fraunhofer ISE
 *
 * This file is part of OpenMUC.
 * For more information visit http://www.openmuc.org
 *
 * OpenMUC is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * OpenMUC is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with OpenMUC.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
package org.openmuc.framework.core.datamanager;

import java.util.List;
import java.util.concurrent.CountDownLatch;

import org.openmuc.framework.data.Flag;
import org.openmuc.framework.data.Record;
import org.openmuc.framework.driver.spi.ChannelRecordContainer;
import org.openmuc.framework.driver.spi.ConnectionException;

public class ReadTask extends DeviceReadTask {

	private final CountDownLatch readTaskFinishedSignal;

	public ReadTask(DataManager dataManager, Device device, List<ChannelRecordContainerImpl> selectedChannels,
			int readTimeout, CountDownLatch readTaskFinishedSignal) {
		super(dataManager, device, selectedChannels, readTimeout);
		this.readTaskFinishedSignal = readTaskFinishedSignal;
	}

	@Override
	@SuppressWarnings("unchecked")
	protected void executeRead() throws UnsupportedOperationException, ConnectionException {
		device.deviceConfig.driverParent.activeDriver.read(device.connection,
				(List<ChannelRecordContainer>) ((List<?>) selectedChannels), true, null, readTimeout);
	}

	@Override
	protected void taskFinished() {
		disabled = true;
		long now = System.currentTimeMillis();
		if (methodNotExceptedExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.setRecord(new Record(null, now, Flag.ACCESS_METHOD_NOT_SUPPORTED));
			}
		}
		else if (unknownDriverExceptionThrown) {
			for (ChannelRecordContainerImpl driverChannel : selectedChannels) {
				driverChannel.setRecord(new Record(null, now, Flag.DRIVER_THREW_UNKNOWN_EXCEPTION));
			}
		}

		readTaskFinishedSignal.countDown();

		synchronized (dataManager.tasksFinished) {
			dataManager.tasksFinished.add(this);
		}
		dataManager.interrupt();
	}

	@Override
	protected void taskAborted() {
		readTaskFinishedSignal.countDown();
	}
}
